def add_repository_flag(repository):
    return '--repository=' + repository


def map_list_with_repository_flag(repositories):
    return tuple(map(
        add_repository_flag,
        repositories
    ))


REPOSITORY_NAMES = (
    'repo-1',
    'r-e-p-o-2',
    'repo3',
)

GROUP_NAMES = (
    'oneandtwo',
    'oneandthree',
    'three',
)

CONFIG_REPOSITORIES = dict({
    REPOSITORY_NAMES[0]: dict({
        'address': 'git@github.com:some-owner/repo-1.git',
        'location': 'git/repo-1',
    }),
    REPOSITORY_NAMES[1]: dict({
        'address': 'git@github.com:other-owner/r-e-p-o-2.git',
        'location': '~/git/r-e-p-o-2',
    }),
    REPOSITORY_NAMES[2]: dict({
        'address': 'git@github.com:owner3/repo3.git',
        'location': '/home/jock/git/repo3',
    }),
})

CONFIG_GROUPS = dict({
    GROUP_NAMES[0]: {'repositories': [REPOSITORY_NAMES[0], REPOSITORY_NAMES[1]]},
    GROUP_NAMES[1]: {'repositories': [REPOSITORY_NAMES[0], REPOSITORY_NAMES[2]]},
    GROUP_NAMES[2]: {'repositories': [REPOSITORY_NAMES[2]]},
})



#!/usr/bin/env python
import os

import yaml

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper


def load_config():
    with open(os.path.expanduser('~/.jockrc'), 'r') as file:
        return yaml.load(file, Loader=Loader)


def get_selected_repositories(selected_repositories, selected_groups):
    config = load_config()

    repositories = dict({})
    config_repositories = config['repositories']
    for repo_name in selected_repositories:
        repositories[repo_name] = config_repositories[repo_name]

    if config.get('groups') is not None:
        config_groups = config['groups']
        for group_name in selected_groups:
            for repo_name in config_groups[group_name]['repositories']:
                repositories[repo_name] = config_repositories[repo_name]

    return repositories



from unittest import TestCase
from unittest.mock import call, patch

from click.testing import CliRunner

from jock.cli import main
from tests.utils import map_list_with_repository_flag, CONFIG_REPOSITORIES, REPOSITORY_NAMES


class TestCLI(TestCase):

    def setUp(self):
        self.runner = CliRunner()

    @patch('jock.cli.get_selected_repositories')
    @patch('jock.cli.git_command')
    def test_git_commands(self, git_command, get_selected_repositories_mock):
        # Given
        get_selected_repositories_mock.return_value = CONFIG_REPOSITORIES
        commands = [
            'clone',
            'add',
            'restore',
            'rm',
            'branch',
            'commit',
            'reset',
            'switch',
            'tag',
            'fetch',
            'pull',
            'push',
            'checkout',
        ]

        args = ('-a', '--woof')
        expected_calls = [
            call(commands[0], CONFIG_REPOSITORIES, args),
            call(commands[1], CONFIG_REPOSITORIES, args),
            call(commands[2], CONFIG_REPOSITORIES, args),
            call(commands[3], CONFIG_REPOSITORIES, args),
            call(commands[4], CONFIG_REPOSITORIES, args),
            call(commands[5], CONFIG_REPOSITORIES, args),
            call(commands[6], CONFIG_REPOSITORIES, args),
            call(commands[7], CONFIG_REPOSITORIES, args),
            call(commands[8], CONFIG_REPOSITORIES, args),
            call(commands[9], CONFIG_REPOSITORIES, args),
            call(commands[10], CONFIG_REPOSITORIES, args),
            call(commands[11], CONFIG_REPOSITORIES, args),
            call(commands[12], CONFIG_REPOSITORIES, args),
        ]
        flagged_repositories = \
            map_list_with_repository_flag(REPOSITORY_NAMES)
        # When
        for command in commands:
            print(flagged_repositories + (command,) + args)
            self.runner.invoke(main, flagged_repositories + (command,) + args)
        # Then
        git_command.assert_has_calls(expected_calls)
        self.assertEqual(git_command.call_count, len(expected_calls))

    @patch('jock.cli.get_selected_repositories')
    @patch('jock.cli.git_command')
    def test_all_repository_flags_work(self, git_mock, get_selected_repositories_mock):
        # Given
        get_selected_repositories_mock.return_value = CONFIG_REPOSITORIES
        flagged_repository_names = (
            '--repository=' + REPOSITORY_NAMES[0],
            '-r ' + REPOSITORY_NAMES[1],
            '-r=' + REPOSITORY_NAMES[2]
        )
        # When
        self.runner.invoke(main, flagged_repository_names + ('clone',))
        # Then
        git_mock.assert_called_once_with('clone', CONFIG_REPOSITORIES, ())



import os
from unittest import TestCase
from unittest.mock import patch

from jock.utils import get_repository_name, get_repository_path


class TestUtils(TestCase):
    def setUp(self):
        self.repository_paths = (
            'git/repo-1',
            '~/git/r-e-p-o-2',
            '/home/jock/git/repo3'
        )

        self.repository_addresses = (
            'git@github.com:some-owner/repo-1.git',
            'git@github.com:other-owner/r-e-p-o-2.git',
            'git@github.com:owner3/repo3.git'
        )

        self.repository_names = (
            'repo-1',
            'r-e-p-o-2',
            'repo3'
        )

    def test_get_repository_name(self):
        for i in range(len(self.repository_addresses)):
            # Given
            expected_name = self.repository_names[i]
            # When
            actual_name = get_repository_name(self.repository_addresses[i])
            # Then
            self.assertEqual(expected_name, actual_name)

    @patch.object(os.path, 'isabs')
    @patch.object(os.path, 'expanduser')
    def test_get_repository_path(self, mock_expanduser, mock_isabs):
        for i in range(len(self.repository_paths)):
            # Given
            mock_expanduser.side_effect = lambda r: '/now/abs/' + r
            mock_isabs.side_effect = lambda _: i % 2 == 0
            expected_path = self.repository_paths[i]
            if i % 2 == 0:
                expected_path = '/now/abs/' + expected_path
            else:
                expected_path = '/now/abs/~/' + expected_path
            # When
            actual_path = get_repository_path(self.repository_paths[i])
            # Then
            self.assertEqual(expected_path, actual_path)



import subprocess
from unittest import TestCase
from unittest.mock import call, patch

import pytest

from jock.git import git_command, git_common
from tests.utils import CONFIG_REPOSITORIES, REPOSITORY_NAMES


class TestGit(TestCase):
    @staticmethod
    def _get_clone_call(name, git_args=()):
        repository = CONFIG_REPOSITORIES[name]
        return call((
                        'git',
                        'clone',
                        repository['address'],
                        repository['location']
                    ) + git_args)

    @patch('jock.git.get_repository_path')
    @patch.object(subprocess, 'run')
    def test_clone_clones_all(self, mock_run, mock_get_repository_path):
        # Given
        mock_get_repository_path.side_effect = lambda r: r
        args = ('--scary', '--bork')
        expected_calls = [
            self._get_clone_call(REPOSITORY_NAMES[0], args),
            self._get_clone_call(REPOSITORY_NAMES[1], args),
            self._get_clone_call(REPOSITORY_NAMES[2], args)
        ]
        # When
        git_command('clone', CONFIG_REPOSITORIES, args)
        # Then
        mock_run.assert_has_calls(expected_calls)
        self.assertEqual(mock_run.call_count, len(REPOSITORY_NAMES))

    @staticmethod
    def _get_common_call(repository_name, command, git_args=()):
        return call(('git', '-C', CONFIG_REPOSITORIES[repository_name]['location'], command) + git_args)

    @patch('jock.git.get_repository_path')
    @patch.object(subprocess, 'run')
    def test_common_call(self, mock_run, mock_get_repository_path):
        # Given
        mock_get_repository_path.side_effect = lambda r: r
        command = 'bark'
        args = ('-a', '--woof')
        expected_calls = [
            self._get_common_call(REPOSITORY_NAMES[0], command, args),
            self._get_common_call(REPOSITORY_NAMES[1], command, args),
            self._get_common_call(REPOSITORY_NAMES[2], command, args)
        ]
        # When
        git_common(command, CONFIG_REPOSITORIES, args)
        # Then
        mock_run.assert_has_calls(expected_calls)
        self.assertEqual(mock_run.call_count, len(REPOSITORY_NAMES))

    @patch('jock.git.get_repository_path')
    @patch.object(subprocess, 'run')
    def test_git_command(self, mock_run, mock_get_repository_path):
        # Given
        mock_get_repository_path.side_effect = lambda r: r
        common_commands = [
            'add',
            'restore',
            'rm',
            'branch',
            'commit',
            'reset',
            'switch',
            'tag',
            'fetch',
            'pull',
            'push',
            'checkout',
        ]
        args = ('-a', '--woof')
        repository_name = REPOSITORY_NAMES[0]
        selected_repositories = dict({repository_name: CONFIG_REPOSITORIES[repository_name]})
        expected_calls = [
            self._get_common_call(repository_name, common_commands[0], args),
            self._get_common_call(repository_name, common_commands[1], args),
            self._get_common_call(repository_name, common_commands[2], args),
            self._get_common_call(repository_name, common_commands[3], args),
            self._get_common_call(repository_name, common_commands[4], args),
            self._get_common_call(repository_name, common_commands[5], args),
            self._get_common_call(repository_name, common_commands[6], args),
            self._get_common_call(repository_name, common_commands[7], args),
            self._get_common_call(repository_name, common_commands[8], args),
            self._get_common_call(repository_name, common_commands[9], args),
            self._get_common_call(repository_name, common_commands[10], args),
            self._get_common_call(repository_name, common_commands[11], args),
            self._get_clone_call(repository_name, args)
        ]
        # When
        for command in common_commands:
            git_command(command, selected_repositories, args)
        git_command('clone', selected_repositories, args)
        # Then
        mock_run.assert_has_calls(expected_calls)
        self.assertEqual(mock_run.call_count, len(expected_calls))

    def test_git_command_exits(self):
        # Given
        unknown_command = 'grrrr'
        expected_exit_code = 1
        # When
        with pytest.raises(SystemExit) as wrapped_exit:
            git_command(unknown_command, CONFIG_REPOSITORIES, ())
        # Then
        self.assertEqual(expected_exit_code, wrapped_exit.value.code)



#!/usr/bin/env python3
from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='git-jock',
    # This will be updated automatically
    version='0.2.1',
    author='Gavin Fenton',
    author_email='contact@gavinfenton.com',
    description='Git helper for multi-repository management',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Version Control',
        'Topic :: Software Development :: Version Control :: Git',
    ],
    python_requires='>=3.7',
)



import subprocess
import sys

import click

from jock.utils import get_repository_path


def git_common(command, selected_repositories, git_args=()):
    for repository_name in selected_repositories:
        repository = selected_repositories[repository_name]
        repository_path = get_repository_path(repository['location'])
        click.echo('Executing [{}] in [{}]'.format(command, repository_path))
        subprocess.run(('git', '-C', repository_path, command) + git_args)


def git_clone(selected_repositories, git_args=()):
    for repository_name in selected_repositories:
        repository = selected_repositories[repository_name]
        repository_path = get_repository_path(repository['location'])
        click.echo(
            'Cloning [{}] in [{}]'.format(repository_name, repository_path)
        )
        subprocess.run(('git', 'clone', repository['address'], repository_path) + git_args)


GIT_COMMANDS = {
    'clone': lambda _, sr, a: git_clone(sr, a),
    'add': git_common,
    'restore': git_common,
    'rm': git_common,
    'branch': git_common,
    'commit': git_common,
    'reset': git_common,
    'switch': git_common,
    'tag': git_common,
    'fetch': git_common,
    'pull': git_common,
    'push': git_common,
    'checkout': git_common,
}


def git_command(command, selected_repositories, git_args):
    git_func = GIT_COMMANDS.get(command)

    if git_func is None:
        print('Unsupported command ' + command)
        sys.exit(1)

    git_func(command, selected_repositories, git_args)



import click

from jock import __version__
from jock.config import get_selected_repositories
from jock.git import git_command

CONFIG_REPOSITORIES = 'config_repositories'
SELECTED_REPOSITORIES = 'selected_repositories'
GIT_ARGS = 'git_args'
CONTEXT_SETTINGS = dict(ignore_unknown_options=True, )


@click.group()
@click.version_option(__version__)
@click.option('--repository', '-r', type=str, multiple=True,
              help='Repository, specified in ~/.jockrc, '
                   'you wish to run commands on. Multiple '
                   'repositories can be specified using '
                   'multiple flags.')
@click.option('--group', '-g', type=str, multiple=True,
              help='Group of repositories, specified in '
                   '~/.jockrc, you wish to run commands on.'
                   'Multiple  groups can be specified using '
                   'multiple flags.')
@click.pass_context
def main(ctx, repository, group):
    ctx.ensure_object(dict)

    repositories = tuple(map(lambda x: x.lstrip(" ="), repository))
    groups = tuple(map(lambda x: x.lstrip(" ="), group))

    ctx.obj[SELECTED_REPOSITORIES] = get_selected_repositories(repositories, groups)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def clone(ctx, git_args):
    git_command('clone', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def add(ctx, git_args):
    git_command('add', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def restore(ctx, git_args):
    git_command('restore', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def rm(ctx, git_args):
    git_command('rm', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def branch(ctx, git_args):
    git_command('branch', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def commit(ctx, git_args):
    git_command('commit', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def reset(ctx, git_args):
    git_command('reset', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def switch(ctx, git_args):
    git_command('switch', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def tag(ctx, git_args):
    git_command('tag', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def fetch(ctx, git_args):
    git_command('fetch', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def pull(ctx, git_args):
    git_command('pull', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def push(ctx, git_args):
    git_command('push', ctx.obj[SELECTED_REPOSITORIES], git_args)


@main.command(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.argument(GIT_ARGS, nargs=-1, required=False)
def checkout(ctx, git_args):
    git_command('checkout', ctx.obj[SELECTED_REPOSITORIES], git_args)


if __name__ == '__main__':
    main(prog_name='jock')



#!/usr/bin/env python
import os
import sys

import jock.cli

if os.environ.get('LC_CTYPE', '') == 'UTF-8':
    os.environ['LC_CTYPE'] = 'en_US.UTF-8'


def main():
    return jock.cli.main()


if __name__ == '__main__':
    sys.exit(main())



import os
import re

REPOSITORY_NAME_PATTERN = '([\\w\\.-]+)(.git)/?$'
REPOSITORY_NAME_PATTERN_GROUP = 1


def get_repository_name(repository):
    match = re.search(REPOSITORY_NAME_PATTERN, repository, re.IGNORECASE)
    if match:
        return match.group(REPOSITORY_NAME_PATTERN_GROUP)


def get_repository_path(path):
    expanded = os.path.expanduser(path)
    if not os.path.isabs(expanded):
        expanded = os.path.expanduser('~/' + path)
    return expanded



import os
from unittest import TestCase
from unittest.mock import patch

import yaml

from jock.config import load_config, get_selected_repositories
from tests.utils import CONFIG_REPOSITORIES, REPOSITORY_NAMES, GROUP_NAMES, CONFIG_GROUPS

open_name = '%s.open' % __name__


class TestConfig(TestCase):
    @patch('builtins.open', read_data='data')
    @patch.object(yaml, 'load')
    @patch.object(os.path, 'expanduser')
    def test_load_config(self, mock_expanduser, mock_yaml, mock_open):
        # Given
        expected_rc_path = '~/.jockrc'
        expected_expanded_path = '/some/path'
        expected_config = dict({'repositories': CONFIG_REPOSITORIES})
        mock_expanduser.return_value = expected_expanded_path
        mock_yaml.return_value = expected_config
        # When
        actual_config = load_config()
        # Then
        mock_expanduser.assert_called_once_with(expected_rc_path)
        mock_open.assert_called_once_with(expected_expanded_path, 'r')
        mock_yaml.assert_called_once()
        self.assertEqual(expected_config, actual_config)

    @patch('jock.config.load_config')
    def test_get_selected_repositories_returns_selected_repos(self, mock_load_config):
        # Given
        selected_repositories = (REPOSITORY_NAMES[0], REPOSITORY_NAMES[2])
        mock_load_config.return_value = dict({
            'repositories': CONFIG_REPOSITORIES,
            'groups': CONFIG_REPOSITORIES
        })
        expected_repositories = dict({
            REPOSITORY_NAMES[0]: CONFIG_REPOSITORIES[REPOSITORY_NAMES[0]],
            REPOSITORY_NAMES[2]: CONFIG_REPOSITORIES[REPOSITORY_NAMES[2]]
        })
        # When
        actual_repositories = get_selected_repositories(selected_repositories, tuple())
        # Then
        mock_load_config.assert_called_once()
        self.assertEqual(expected_repositories, actual_repositories)

    @patch('jock.config.load_config')
    def test_get_selected_repositories_returns_selected_groups(self, mock_load_config):
        # Given
        selected_groups = (GROUP_NAMES[1], GROUP_NAMES[2])
        mock_load_config.return_value = dict({
            'repositories': CONFIG_REPOSITORIES,
            'groups': CONFIG_GROUPS
        })
        expected_repositories = dict({
            REPOSITORY_NAMES[0]: CONFIG_REPOSITORIES[REPOSITORY_NAMES[0]],
            REPOSITORY_NAMES[2]: CONFIG_REPOSITORIES[REPOSITORY_NAMES[2]]
        })
        # When
        actual_repositories = get_selected_repositories(tuple(), selected_groups)
        # Then
        mock_load_config.assert_called_once()
        self.assertEqual(expected_repositories, actual_repositories)

    @patch('jock.config.load_config')
    def test_get_selected_repositories_returns_both_selections(self, mock_load_config):
        # Given
        selected_repositories = (REPOSITORY_NAMES[1],)
        selected_groups = (GROUP_NAMES[1], GROUP_NAMES[2])
        mock_load_config.return_value = dict({
            'repositories': CONFIG_REPOSITORIES,
            'groups': CONFIG_GROUPS
        })
        expected_repositories = dict({
            REPOSITORY_NAMES[0]: CONFIG_REPOSITORIES[REPOSITORY_NAMES[0]],
            REPOSITORY_NAMES[1]: CONFIG_REPOSITORIES[REPOSITORY_NAMES[1]],
            REPOSITORY_NAMES[2]: CONFIG_REPOSITORIES[REPOSITORY_NAMES[2]]
        })
        # When
        actual_repositories = get_selected_repositories(selected_repositories, selected_groups)
        # Then
        mock_load_config.assert_called_once()
        self.assertEqual(expected_repositories, actual_repositories)
